// alpha.c : Defines the entry point for the application.
//
// Author: Feng Yuan [http://www.fengyuan.com/article/alphablend.html]
//

#include <windows.h>
#include "resource.h"

#pragma lib <msimg32.lib>

#define MAX_LOADSTRING 100

// Global Variables:
HINSTANCE hInst;								// current instance
TCHAR szTitle[MAX_LOADSTRING];					// The title bar text
TCHAR szWindowClass[MAX_LOADSTRING];			// The title bar text

// Foward declarations of functions included in this code module:
ATOM				MyRegisterClass(HINSTANCE hInstance);
BOOL				InitInstance(HINSTANCE, int);
LRESULT CALLBACK	WndProc(HWND, UINT, WPARAM, LPARAM);

char szFileName[MAX_PATH];

int APIENTRY WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
 	// TODO: Place code here.
	MSG msg;
	HACCEL hAccelTable;

    if ( lpCmdLine )
        strcpy(szFileName, lpCmdLine);
    else
        szFileName[0] = 0;

	// Initialize global strings
	LoadString(hInstance, IDS_APP_TITLE, szTitle, MAX_LOADSTRING);
	LoadString(hInstance, IDC_ALPHA, szWindowClass, MAX_LOADSTRING);
	MyRegisterClass(hInstance);

	// Perform application initialization:
	if (!InitInstance (hInstance, nCmdShow))
	{
		return FALSE;
	}

	hAccelTable = LoadAccelerators(hInstance, (LPCTSTR)IDC_ALPHA);

	// Main message loop:
	while (GetMessage(&msg, NULL, 0, 0))
	{
		if (!TranslateAccelerator(msg.hwnd, hAccelTable, &msg))
		{
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
	}

	return msg.wParam;
}

//
//  FUNCTION: MyRegisterClass()
//
//  PURPOSE: Registers the window class.
//
//  COMMENTS:
//
//    This function and its usage is only necessary if you want this code
//    to be compatible with Win32 systems prior to the 'RegisterClassEx'
//    function that was added to Windows 95. It is important to call this function
//    so that the application will get 'well formed' small icons associated
//    with it.
//
ATOM MyRegisterClass(HINSTANCE hInstance)
{
	WNDCLASSEX wcex;

	wcex.cbSize = sizeof(WNDCLASSEX);

	wcex.style			= CS_HREDRAW | CS_VREDRAW;
	wcex.lpfnWndProc	= (WNDPROC)WndProc;
	wcex.cbClsExtra		= 0;
	wcex.cbWndExtra		= 0;
	wcex.hInstance		= hInstance;
	wcex.hIcon			= LoadIcon(hInstance, (LPCTSTR)IDI_ALPHA);
	wcex.hCursor		= LoadCursor(NULL, IDC_ARROW);
	wcex.hbrBackground	= (HBRUSH)(COLOR_WINDOW+1);
	wcex.lpszMenuName	= (LPCSTR)IDC_ALPHA;
	wcex.lpszClassName	= szWindowClass;
	wcex.hIconSm		= LoadIcon(wcex.hInstance, (LPCTSTR)IDI_SMALL);

	return RegisterClassEx(&wcex);
}

//
//   FUNCTION: InitInstance(HANDLE, int)
//
//   PURPOSE: Saves instance handle and creates main window
//
//   COMMENTS:
//
//        In this function, we save the instance handle in a global variable and
//        create and display the main program window.
//
BOOL InitInstance(HINSTANCE hInstance, int nCmdShow)
{
   HWND hWnd;

   hInst = hInstance; // Store instance handle in our global variable

   hWnd = CreateWindow(szWindowClass, szTitle, WS_OVERLAPPEDWINDOW,
      CW_USEDEFAULT, 0, CW_USEDEFAULT, 0, NULL, NULL, hInstance, NULL);

   if (!hWnd)
   {
      return FALSE;
   }

   ShowWindow(hWnd, nCmdShow);
   UpdateWindow(hWnd);

   return TRUE;
}

#pragma pack(push, 1)

typedef struct
{
    BYTE    IDLength;
    BYTE    ColorMapType;
    BYTE    ImageType;
    WORD    CMapStart;
    WORD    CMapLength;
    BYTE    CMapDepth;
    WORD    XOffset;
    WORD    YOffset;
    WORD    Width;
    WORD    Height;
    BYTE    PixelDepth;
    BYTE    ImageDescriptor;
}   TGA_Header;

#pragma pack(pop)

HBITMAP Load32bppTga(const TCHAR * pFileName, BOOL bPreMultiply)
{
	HANDLE handle = CreateFile(pFileName, GENERIC_READ, FILE_SHARE_READ,
		NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);

	if ( handle == INVALID_HANDLE_VALUE )
		return NULL;

    TGA_Header header;

	DWORD dwRead = 0;
	ReadFile(handle, &header, sizeof(header), &dwRead, NULL);

    if ( (header.IDLength!=0) || (header.ColorMapType!=0) || (header.ImageType!=2) ||
         (header.PixelDepth!=32) || (header.ImageDescriptor!=8) )
    {
        CloseHandle(handle);
        return NULL;
    }

	BITMAPINFO bmp;
	memset(&bmp, 0, sizeof(BITMAPINFO));
	bmp.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
	bmp.bmiHeader.biWidth = header.Width;
	bmp.bmiHeader.biHeight = header.Height;
	bmp.bmiHeader.biPlanes = 1;
	bmp.bmiHeader.biBitCount = 32;

    void * pBits = NULL;

    HBITMAP hBmp = CreateDIBSection(NULL, &bmp, DIB_RGB_COLORS, &pBits, NULL, 0);

    if ( hBmp==NULL )
    {
        CloseHandle(handle);
        return NULL;
    }

	ReadFile(handle, pBits, header.Width * header.Height * 4, &dwRead, NULL);

    CloseHandle(handle);

    if ( bPreMultiply )
    {
        for (int y=0; y<header.Height; y++)
        {
            BYTE * pPixel = (BYTE *) pBits + header.Width * 4 * y;

            for (int x=0; x<header.Width; x++)
            {
                pPixel[0] = pPixel[0] * pPixel[3] / 255;
                pPixel[1] = pPixel[1] * pPixel[3] / 255;
                pPixel[2] = pPixel[2] * pPixel[3] / 255;

                pPixel += 4;
            }
        }
    }

    return hBmp;
}


void AlphaDraw(HDC hDC, int x, int y, int width, int height, HBITMAP hBmp)
{
    HDC     hMemDC = CreateCompatibleDC(hDC);
    HGDIOBJ hOld   = SelectObject(hMemDC, hBmp);


    HBRUSH hBrush = CreateSolidBrush(RGB(0xFF, 0xFF, 0));
    SelectObject(hDC, hBrush);
    Ellipse(hDC, x, y, width*2, height * 2);
    SelectObject(hDC, GetStockObject(WHITE_BRUSH));
    DeleteObject(hBrush);

    BitBlt(hDC, x, y, width, height, hMemDC, 0, 0, SRCCOPY);

    BLENDFUNCTION pixelblend = { AC_SRC_OVER, 0, 255, AC_SRC_ALPHA };

    AlphaBlend(hDC, x, y+height, width, height, hMemDC, 0, 0, width, height, pixelblend);

    BLENDFUNCTION blend50 = { AC_SRC_OVER, 0, 128, 0 };

    AlphaBlend(hDC, x+width, y, width, height, hMemDC, 0, 0, width, height, blend50);

    SelectObject(hMemDC, hOld);
    DeleteObject(hMemDC);
}


void OnDraw(HDC hDC, const char * pFileName)
{
    static HBITMAP hBmp = NULL;

    if ( hBmp==NULL )
        hBmp = Load32bppTga(pFileName, TRUE);

    BITMAP bmp;
    GetObject(hBmp, sizeof(BITMAP), &bmp);

    AlphaDraw(hDC, 10, 10, bmp.bmWidth, bmp.bmHeight, hBmp);
}


LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	PAINTSTRUCT ps;

	switch (message)
	{
		case WM_COMMAND:
			// Parse the menu selections:
			switch ( LOWORD(wParam) )
			{
				case IDM_EXIT:
				   DestroyWindow(hWnd);
				   break;
				default:
				   return DefWindowProc(hWnd, message, wParam, lParam);
			}
			break;

        case WM_PAINT:
			OnDraw(BeginPaint(hWnd, &ps), "small.tga"); // szFileName);
			EndPaint(hWnd, &ps);
			break;

        case WM_DESTROY:
			PostQuitMessage(0);
			break;
		default:
			return DefWindowProc(hWnd, message, wParam, lParam);
   }
   return 0;
}

